@@ -188,16 +188,19 @@ typedef enum _GstQuarkId
   GST_QUARK_EVENT_SEGMENT_DONE = 159,
   GST_QUARK_EVENT_STREAM_START = 160,
   GST_QUARK_STREAM_ID = 161,
-  GST_QUARK_EVENT_CONTEXT = 162,
-  GST_QUARK_QUERY_CONTEXT = 163,
-  GST_QUARK_MESSAGE_NEED_CONTEXT = 164,
-  GST_QUARK_MESSAGE_HAVE_CONTEXT = 165,
-  GST_QUARK_CONTEXT = 166,
-  GST_QUARK_CONTEXT_TYPES = 167,
-  GST_QUARK_MESSAGE_STREAM_START = 168,
-  GST_QUARK_GROUP_ID = 169,
-  GST_QUARK_URI_REDIRECTION = 170,
-  GST_QUARK_MAX = 171
+  GST_QUARK_QUERY_CONTEXT = 162,
+  GST_QUARK_MESSAGE_NEED_CONTEXT = 163,
+  GST_QUARK_MESSAGE_HAVE_CONTEXT = 164,
+  GST_QUARK_CONTEXT = 165,
+  GST_QUARK_CONTEXT_TYPE = 166,
+  GST_QUARK_MESSAGE_STREAM_START = 167,
+  GST_QUARK_GROUP_ID = 168,
+  GST_QUARK_URI_REDIRECTION = 169,
+  GST_QUARK_MESSAGE_DEVICE_ADDED = 170,
+  GST_QUARK_MESSAGE_DEVICE_REMOVED = 171,
+  GST_QUARK_DEVICE = 172,
+  GST_QUARK_URI_REDIRECTION_PERMANENT = 173,
+  GST_QUARK_MAX = 174
 } GstQuarkId;
 
 extern GQuark _priv_gst_quark_table[GST_QUARK_MAX];
