@@ -47,8 +47,8 @@ typedef struct _GstStructure GstStructure;
  * A function that will be called in gst_structure_foreach(). The function may
  * not modify @value.
  *
- * Returns: TRUE if the foreach operation should continue, FALSE if
- * the foreach operation should stop with FALSE.
+ * Returns: %TRUE if the foreach operation should continue, %FALSE if
+ * the foreach operation should stop with %FALSE.
  */
 typedef gboolean (*GstStructureForeachFunc) (GQuark   field_id,
                                              const GValue * value,
@@ -63,8 +63,8 @@ typedef gboolean (*GstStructureForeachFunc) (GQuark   field_id,
  * A function that will be called in gst_structure_map_in_place(). The function
  * may modify @value.
  *
- * Returns: TRUE if the map operation should continue, FALSE if
- * the map operation should stop with FALSE.
+ * Returns: %TRUE if the map operation should continue, %FALSE if
+ * the map operation should stop with %FALSE.
  */
 typedef gboolean (*GstStructureMapFunc)     (GQuark   field_id,
                                              GValue * value,
@@ -101,6 +101,8 @@ GstStructure *        gst_structure_new_id               (GQuark name_quark,
                                                           GQuark field_quark,
                                                           ...) G_GNUC_MALLOC;
 
+GstStructure *        gst_structure_new_from_string      (const gchar * string);
+
 GstStructure *        gst_structure_copy                 (const GstStructure  * structure) G_GNUC_MALLOC;
 
 gboolean              gst_structure_set_parent_refcount  (GstStructure        * structure,
@@ -228,6 +230,14 @@ gboolean              gst_structure_get_uint             (const GstStructure  *
                                                           const gchar         * fieldname,
                                                           guint               * value);
 
+gboolean              gst_structure_get_int64            (const GstStructure  * structure,
+                                                          const gchar         * fieldname,
+                                                          gint64              * value);
+
+gboolean              gst_structure_get_uint64           (const GstStructure  * structure,
+                                                          const gchar         * fieldname,
+                                                          guint64             * value);
+
 gboolean              gst_structure_get_double           (const GstStructure  * structure,
                                                           const gchar         * fieldname,
                                                           gdouble             * value);
