@@ -340,6 +340,9 @@ gst_capsfilter_prepare_buf (GstBaseTransform * trans, GstBuffer * input,
         } else {
           ret = GST_FLOW_NOT_NEGOTIATED;
         }
+      } else {
+        gst_capsfilter_push_pending_events (filter, pending_events);
+        pending_events = NULL;
       }
 
       g_list_free_full (pending_events, (GDestroyNotify) gst_event_unref);
@@ -382,7 +385,8 @@ gst_capsfilter_sink_event (GstBaseTransform * trans, GstEvent * event)
     GList *l;
 
     for (l = filter->pending_events; l; l = l->next) {
-      if (GST_EVENT_TYPE (l->data) == GST_EVENT_SEGMENT) {
+      if (GST_EVENT_TYPE (l->data) == GST_EVENT_SEGMENT ||
+          GST_EVENT_TYPE (l->data) == GST_EVENT_EOS) {
         gst_event_unref (l->data);
         filter->pending_events = g_list_delete_link (filter->pending_events, l);
         break;
