@@ -36,10 +36,6 @@ struct TestData
 static void
 setup_test_objects (struct TestData *td, GstPadChainFunction chain_func)
 {
-  GstSegment segment;
-
-  gst_segment_init (&segment, GST_FORMAT_BYTES);
-
   td->mycaps = gst_caps_new_empty_simple ("test/test");
 
   td->funnel = gst_element_factory_make ("funnel", NULL);
@@ -64,16 +60,13 @@ setup_test_objects (struct TestData *td, GstPadChainFunction chain_func)
 
   td->mysrc1 = gst_pad_new ("src1", GST_PAD_SRC);
   gst_pad_set_active (td->mysrc1, TRUE);
-
-  gst_pad_push_event (td->mysrc1, gst_event_new_stream_start ("test"));
-  gst_pad_set_caps (td->mysrc1, td->mycaps);
-  gst_pad_push_event (td->mysrc1, gst_event_new_segment (&segment));
+  gst_check_setup_events_with_stream_id (td->mysrc1, td->funnel, td->mycaps,
+      GST_FORMAT_BYTES, "test1");
 
   td->mysrc2 = gst_pad_new ("src2", GST_PAD_SRC);
   gst_pad_set_active (td->mysrc2, TRUE);
-  gst_pad_push_event (td->mysrc2, gst_event_new_stream_start ("test"));
-  gst_pad_set_caps (td->mysrc2, td->mycaps);
-  gst_pad_push_event (td->mysrc2, gst_event_new_segment (&segment));
+  gst_check_setup_events_with_stream_id (td->mysrc2, td->funnel, td->mycaps,
+      GST_FORMAT_BYTES, "test2");
 
   fail_unless (GST_PAD_LINK_SUCCESSFUL (gst_pad_link (td->funnelsrc,
               td->mysink)));
@@ -126,10 +119,6 @@ chain_ok (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 GST_START_TEST (test_funnel_simple)
 {
   struct TestData td;
-#if 0
-  GstBuffer *buf1 = NULL;
-  GstBuffer *buf2 = NULL;
-#endif
 
   setup_test_objects (&td, chain_ok);
 
@@ -141,18 +130,6 @@ GST_START_TEST (test_funnel_simple)
 
   fail_unless (bufcount == 2);
 
-#if 0
-  fail_unless (gst_pad_alloc_buffer (td.mysrc1, 0, 1024, td.mycaps,
-          &buf1) == GST_FLOW_OK);
-  fail_unless (gst_pad_alloc_buffer (td.mysrc2, 1024, 1024, td.mycaps,
-          &buf2) == GST_FLOW_OK);
-
-  fail_unless (alloccount == 2);
-
-  gst_buffer_unref (buf1);
-  gst_buffer_unref (buf2);
-#endif
-
   release_test_objects (&td);
 }
 
@@ -235,6 +212,28 @@ GST_START_TEST (test_funnel_eos)
   gst_object_unref (td.funnelsink11);
   fail_unless (num_eos == 2);
 
+  /* send only eos to check, it handles empty streams */
+  td.funnelsink11 = gst_element_get_request_pad (td.funnel, "sink_11");
+  fail_unless (td.funnelsink11 != NULL);
+  fail_unless (!strcmp (GST_OBJECT_NAME (td.funnelsink11), "sink_11"));
+
+  fail_unless (GST_PAD_LINK_SUCCESSFUL (gst_pad_link (td.mysrc1,
+              td.funnelsink11)));
+
+  fail_unless (gst_pad_push_event (td.mysrc1, gst_event_new_flush_start ()));
+  fail_unless (gst_pad_push_event (td.mysrc1, gst_event_new_flush_stop (TRUE)));
+  fail_unless (gst_pad_push_event (td.mysrc2, gst_event_new_flush_start ()));
+  fail_unless (gst_pad_push_event (td.mysrc2, gst_event_new_flush_stop (TRUE)));
+
+  fail_unless (gst_pad_push_event (td.mysrc1, gst_event_new_eos ()));
+  fail_unless (gst_pad_push_event (td.mysrc2, gst_event_new_eos ()));
+  fail_unless (num_eos == 3);
+
+  fail_unless (gst_pad_unlink (td.mysrc1, td.funnelsink11));
+  gst_element_release_request_pad (td.funnel, td.funnelsink11);
+  gst_object_unref (td.funnelsink11);
+  fail_unless (num_eos == 3);
+
   td.funnelsink11 = gst_element_get_request_pad (td.funnel, "sink_11");
   fail_unless (td.funnelsink11 != NULL);
   fail_unless (!strcmp (GST_OBJECT_NAME (td.funnelsink11), "sink_11"));
